/*
 * Copyright (c) IntellectualCrafters - 2014. You are not allowed to distribute
 * and/or monetize any of our intellectual property. IntellectualCrafters is not
 * affiliated with Mojang AB. Minecraft is a trademark of Mojang AB.
 *
 * >> File = Clear.java >> Generated by: Citymonstret at 2014-08-09 01:41
 */

package com.intellectualcrafters.plot.commands;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import com.intellectualcrafters.plot.C;
import com.intellectualcrafters.plot.PlayerFunctions;
import com.intellectualcrafters.plot.Plot;
import com.intellectualcrafters.plot.PlotComment;
import com.intellectualcrafters.plot.PlotMain;
import com.intellectualcrafters.plot.database.DBFunc;

/**
 * Created by Citymonstret on 2014-08-01.
 */
public class Inbox extends SubCommand {

    public Inbox() {
        super(Command.INBOX, "Review a the comments for a plot", "comment", CommandCategory.ACTIONS, true);
    }

    @Override
    public boolean execute(final Player plr, final String... args) {
        if (!PlayerFunctions.isInPlot(plr)) {
            PlayerFunctions.sendMessage(plr, C.NOT_IN_PLOT);
            return false;
        }
        final Plot plot = PlayerFunctions.getCurrentPlot(plr);
        if (!plot.hasOwner()) {
            PlayerFunctions.sendMessage(plr, C.NOT_IN_PLOT);
            return false;
        }

        Integer tier = null;
        final UUID uuid = plr.getUniqueId();
        if (PlotMain.hasPermission(plr, "plots.admin")) {
            tier = 0;
        }
        else if (plot.owner == uuid) {
            tier = 1;
        }
        else if (plot.helpers.contains(uuid)) {
            tier = 2;
        }
        else if (plot.trusted.contains(uuid)) {
            tier = 3;
        }
        else {
            tier = 4;
        }

        if (args.length > 0) {
            switch (args[0].toLowerCase()) {
                case "admin":
                    if (tier <= 0) {
                        tier = 0;
                    }
                    else {
                        PlayerFunctions.sendMessage(plr, C.NO_PERM_INBOX);
                        return false;
                    }
                    break;
                case "owner":
                    if (tier <= 1) {
                        tier = 1;
                    }
                    else {
                        PlayerFunctions.sendMessage(plr, C.NO_PERM_INBOX);
                        return false;
                    }
                    break;
                case "helper":
                    if (tier <= 2) {
                        tier = 2;
                    }
                    else {
                        PlayerFunctions.sendMessage(plr, C.NO_PERM_INBOX);
                        return false;
                    }
                    break;
                case "trusted":
                    if (tier <= 3) {
                        tier = 3;
                    }
                    else {
                        PlayerFunctions.sendMessage(plr, C.NO_PERM_INBOX);
                        return false;
                    }
                    break;
                case "everyone":
                    if (tier <= 4) {
                        tier = 4;
                    }
                    else {
                        PlayerFunctions.sendMessage(plr, C.NO_PERM_INBOX);
                        return false;
                    }
                    break;
                case "default":
                    PlayerFunctions.sendMessage(plr, C.INVALID_INBOX, Arrays.copyOfRange(new String[] { "admin", "owner", "helper", "trusted", "everyone" }, tier, 4));
                    return false;
            }
        }

        final String world = plr.getWorld().getName();
        final int tier2 = tier;

        Bukkit.getScheduler().runTaskAsynchronously(PlotMain.getMain(), new Runnable() {
            @Override
            public void run() {
                ArrayList<PlotComment> comments = plot.settings.getComments(tier2);
                if (comments == null) {
                    comments = DBFunc.getCommenst(world, plot, tier2);
                    plot.settings.setComments(comments);
                }

                if (args.length == 2) {
                    final String[] split = args[1].toLowerCase().split(":");
                    if (!split[0].equals("clear")) {
                        PlayerFunctions.sendMessage(plr, "&c/plot inbox [tier] [clear][:#]");
                        return;
                    }
                    if (split.length > 1) {
                        try {
                            final int index = Integer.parseInt(split[1]);
                            final PlotComment comment = comments.get(index - 1);
                            DBFunc.removeComment(world, plot, comment);
                            plot.settings.removeComment(comment);        
                            PlayerFunctions.sendMessage(plr, C.COMMENT_REMOVED, "1 comment");
                            return;
                        }
                        catch (final Exception e) {
                            PlayerFunctions.sendMessage(plr, "&cInvalid index:\n/plot inbox [tier] [clear][:#]");
                            return;
                        }
                    }
                    for (final PlotComment comment : comments) {
                        DBFunc.removeComment(world, plot, comment);
                    }
                    plot.settings.removeComments(comments);
                    PlayerFunctions.sendMessage(plr, C.COMMENT_REMOVED, "all comments in that category");
                    return;
                }
                else {
                    final List<String> recipients = Arrays.asList(new String[] { "A", "O", "H", "T", "E" });
                    int count = 1;
                    final StringBuilder message = new StringBuilder();
                    String prefix = "";
                    for (final PlotComment comment : comments) {
                        message.append(prefix + "[" + count + "]&6[&c" + recipients.get(tier2) + "&6] &7" + comment.senderName + "&f: " + comment.comment);
                        prefix = "\n";
                        count++;
                    }
                    if (comments.size() == 0) {
                        message.append("&cNo messages.");
                    }
                    PlayerFunctions.sendMessage(plr, message.toString());
                }
            }
        });
        return true;
    }
}
